@@ -6,22 +6,36 @@
 # Antarctic-plots (https://github.com/mdtanker/antarctic_plots)
 #
 
-from typing import TYPE_CHECKING, Union
+from typing import TYPE_CHECKING, Callable, Union
 
+import matplotlib as mpl
 import matplotlib.pyplot as plt
 import numpy as np
 import pandas as pd
 import pygmt
 import pyogrio
+import rioxarray  # noqa
 import verde as vd
 import xarray as xr
 from pyproj import Transformer
 
-from antarctic_plots import maps
+from antarctic_plots import fetch, maps
 
 if TYPE_CHECKING:
     import geopandas as gpd
-# import seaborn as sns
+
+try:
+    import seaborn as sns
+except ImportError:
+    _has_seaborn = False
+else:
+    _has_seaborn = True
+
+
+# function to give RMSE of data
+def RMSE(data):
+    return np.sqrt(np.nanmedian(data**2).item())
+    # return np.sqrt(np.nanmean(data**2).item())
 
 
 def get_grid_info(grid):
@@ -43,35 +57,58 @@ def get_grid_info(grid):
         grid registration)
     """
 
-    if isinstance(grid, str):
-        # grid = xr.load_dataarray(grid)
-        try:
-            # grid = pygmt.load_dataarray(grid)
-            grid = xr.load_dataarray(grid)
-        except ValueError:
-            print("getting grid info didnt work")
-            pass
-            # grid = xr.open_rasterio(grid)
-            # grid = rioxarray.open_rasterio(grid)
+    # if isinstance(grid, str):
+    # grid = xr.load_dataarray(grid)
+    # try:
+    # grid = xr.load_dataarray(grid).squeeze()
+    # except ValueError:
+    # print("loading grid as dataarray didn't work")
+    # raise
+    # pass
+    # grid = xr.open_rasterio(grid)
+    # grid = rioxarray.open_rasterio(grid)
+    if isinstance(grid, xr.DataArray):
+        if int(len(grid.dims)) > 2:
+            grid = grid.squeeze()
 
-    if int(len(grid.dims)) > 2:
-        grid = grid.squeeze()
+    try:
+        spacing = float(pygmt.grdinfo(grid, per_column="n", o=7)[:-1])
+    except Exception:  # pygmt.exceptions.GMTInvalidInput:
+        print("grid spacing can't be extracted")
+        spacing = None
 
     try:
-        spacing = pygmt.grdinfo(grid, per_column="n", o=7)[:-1]
         region = [
             float(pygmt.grdinfo(grid, per_column="n", o=i)[:-1]) for i in range(4)
         ]
+    except Exception:  # pygmt.exceptions.GMTInvalidInput:
+        print("grid region can't be extracted")
+        region = None
+
+    try:
         zmin = float(pygmt.grdinfo(grid, per_column="n", o=4)[:-1])
+    except Exception:  # pygmt.exceptions.GMTInvalidInput:
+        print("grid zmin can't be extracted")
+        zmin = None
+
+    try:
         zmax = float(pygmt.grdinfo(grid, per_column="n", o=5)[:-1])
+    except Exception:  # pygmt.exceptions.GMTInvalidInput:
+        print("grid zmax can't be extracted")
+        zmax = None
+
+    try:
+        reg = grid.gmt.registration
+        registration = "g" if reg == 0 else "p"
+    except AttributeError:
+        print(
+            "grid registration not extracted, re-trying with file loaded as xarray grid"
+        )  # noqa
+        grid = xr.load_dataarray(grid)
         reg = grid.gmt.registration
         registration = "g" if reg == 0 else "p"
     except Exception:
-        print("grid info can't be extracted, check number of dimensions, should be 2.")
-        spacing = None
-        region = None
-        zmin = None
-        zmax = None
+        print("grid registration can't be extracted")
         registration = None
 
     return spacing, region, zmin, zmax, registration
@@ -171,21 +208,22 @@ def epsg3031_to_latlon(df, reg: bool = False, input=["x", "y"], output=["lon", "
 
     transformer = Transformer.from_crs("epsg:3031", "epsg:4326")
 
+    df_out = df.copy()
+
     if isinstance(df, list):
-        xy = df.copy()
-        df = list(transformer.transform(xy[0], xy[1]))
+        df_out = list(transformer.transform(df_out[0], df_out[1]))
     else:
-        df[output[1]], df[output[0]] = transformer.transform(
-            df[input[0]].tolist(), df[input[1]].tolist()
+        df_out[output[1]], df_out[output[0]] = transformer.transform(
+            df_out[input[0]].tolist(), df_out[input[1]].tolist()
         )
         if reg is True:
-            df = [
-                df[output[0]].min(),
-                df[output[0]].max(),
-                df[output[1]].min(),
-                df[output[1]].max(),
+            df_out = [
+                df_out[output[0]].min(),
+                df_out[output[0]].max(),
+                df_out[output[1]].min(),
+                df_out[output[1]].max(),
             ]
-    return df
+    return df_out
 
 
 def reg_str_to_df(input, names=["x", "y"], reverse=False):
@@ -255,11 +293,13 @@ def GMT_reg_xy_to_ll(input, decimal_degree=False):
 def GMT_reg_to_bounding_box(input):
     """
     Convert GMT region string [e, w, n, s] to bounding box format used for icepyx:
-    [ lower left long,
-      lower left lat,
-      upper right long,
-      uper right lat
+    [ lower left longitude,
+      lower left latitude,
+      upper right longitude,
+      uper right latitude
     ]
+    Same format as [xmin, ymin, xmax, ymax], used for `bbox` parameter of
+    geopandas.read_file
 
     Parameters
     ----------
@@ -268,7 +308,7 @@ def GMT_reg_to_bounding_box(input):
 
     Returns
     -------
-    np.ndarray
+    list
         Array of 4 strings in bounding box format.
     """
     return [input[0], input[2], input[1], input[3]]
@@ -294,6 +334,80 @@ def region_to_bounding_box(input):
     return box
 
 
+def points_inside_region(
+    df: pd.DataFrame,
+    region: list,
+    names: list = ["x", "y"],
+):
+    """
+    return a subset of a dataframe which is within a region
+
+    Parameters
+    ----------
+    df : pd.DataFrame
+        dataframe with columns 'x','y' to use for defining if within region
+    region : list
+        GMT region string to use as bounds for new subset dataframe
+
+    Returns
+    -------
+    pd.DataFrame
+       returns a subset dataframe
+    """
+    # make a copy of the dataframe
+    df1 = df.copy()
+
+    # make column of booleans for whether row is within the region
+    df1["inside"] = vd.inside(coordinates=(df1[names[0]], df1[names[1]]), region=region)
+
+    # subset if True
+    df_inside = df1.loc[df1.inside == True].copy()  # noqa
+
+    # drop the column 'inside'
+    df_inside.drop(columns="inside", inplace=True)
+
+    return df_inside
+
+
+def block_reduce(
+    df: pd.DataFrame,
+    reduction: Callable,
+    input_coord_names: list = ["x", "y"],
+    input_data_names: list = None,
+    **kwargs,
+):
+    # define verde reducer function
+    reducer = vd.BlockReduce(reduction, **kwargs)
+
+    # if no data names provided, use all columns
+    if input_data_names is None:
+        input_data_names = list(df.columns.drop(input_coord_names))
+
+    # get tuples of pd.Series
+    input_coords = tuple([df[col] for col in input_coord_names])
+    input_data = tuple([df[col] for col in input_data_names])
+
+    # apply reduction
+    coordinates, data = reducer.filter(
+        coordinates=input_coords,
+        data=input_data,
+    )
+
+    # add reduced coordinates to a dictionary
+    coord_cols = {k: v for k, v in zip(input_coord_names, coordinates)}
+
+    # add reduced data to a dictionary
+    if len(input_data_names) < 2:
+        data_cols = {input_data_names[0]: data}
+    else:
+        data_cols = {k: v for k, v in zip(input_data_names, data)}
+
+    # merge dicts and create dataframe
+    df_reduced = pd.DataFrame(data=coord_cols | data_cols)
+
+    return df_reduced
+
+
 def mask_from_shp(
     shapefile: Union[str or gpd.geodataframe.GeoDataFrame],
     invert: bool = True,
@@ -303,6 +417,8 @@ def mask_from_shp(
     spacing=None,
     masked: bool = False,
     crs: str = "epsg:3031",
+    pixel_register=True,
+    input_coord_names=("x", "y"),
 ):
     """
     Create a mask or a masked grid from area inside or outside of a closed shapefile.
@@ -340,31 +456,44 @@ def mask_from_shp(
     """
 
     if isinstance(shapefile, str):
-        # shp = gpd.read_file(shapefile).geometry
         shp = pyogrio.read_dataframe(shapefile)
     else:
         shp = shapefile
 
     if xr_grid is None and grid_file is None:
         coords = vd.grid_coordinates(
-            region=region, spacing=spacing, pixel_register=True
+            region=region,
+            spacing=spacing,
+            pixel_register=pixel_register,
         )
         ds = vd.make_xarray_grid(
             coords, np.ones_like(coords[0]), dims=("y", "x"), data_names="z"
         )
         xds = ds.z.rio.write_crs(crs)
     elif xr_grid is not None:
-        xds = xr_grid.rio.write_crs(crs)
+        xds = xr_grid.rio.write_crs(crs).rio.set_spatial_dims(
+            input_coord_names[0], input_coord_names[1]
+        )
     elif grid_file is not None:
-        xds = xr.load_dataarray(grid_file).rio.write_crs(crs)
+        xds = (
+            xr.load_dataarray(grid_file)
+            .rio.write_crs(crs)
+            .rio.set_spatial_dims(input_coord_names[0], input_coord_names[1])
+        )
 
-    masked_grd = xds.rio.clip(shp.geometry, xds.rio.crs, drop=False, invert=invert)
+    masked_grd = xds.rio.clip(
+        shp.geometry,
+        xds.rio.crs,
+        drop=False,
+        invert=invert,
+    )
     mask_grd = np.isfinite(masked_grd)
 
     if masked is True:
         output = masked_grd
     elif masked is False:
         output = mask_grd
+
     return output
 
 
@@ -398,7 +527,7 @@ def alter_region(
     Returns
     -------
     list
-        Returns of list of the following variables (region, buffer_region, proj)
+        Returns of list of the following variables: region, buffer_region
     """
     E, W = starting_region[0], starting_region[1]
     N, S = starting_region[2], starting_region[3]
@@ -428,9 +557,10 @@ def alter_region(
 def set_proj(
     region: Union[str or np.ndarray],
     fig_height: float = 15,
+    fig_width: float = None,
 ) -> str:
     """
-    Gives GMT format projection string from region and figure height.
+    Gives GMT format projection string from region and figure height or width.
     Inspired from https://github.com/mrsiegfried/Venturelli2020-GRL.
 
     Parameters
@@ -439,6 +569,9 @@ def set_proj(
         GMT-format region str or list (e, w, n, s) in meters EPSG:3031
     fig_height : float
         desired figure height in cm
+    fig_width : float
+        instead of using figure height, set the projection based on figure width in cm,
+        by default is None
 
     Returns
     -------
@@ -447,9 +580,14 @@ def set_proj(
         fig_height)
     """
     e, w, n, s = region
-    fig_width = fig_height * (w - e) / (s - n)
 
-    ratio = (s - n) / (fig_height / 100)
+    if fig_width is not None:
+        fig_height = fig_width * (s - n) / (w - e)
+        ratio = (w - e) / (fig_width / 100)
+    else:
+        fig_width = fig_height * (w - e) / (s - n)
+        ratio = (s - n) / (fig_height / 100)
+
     proj = f"x1:{ratio}"
     proj_latlon = f"s0/-90/-71/1:{ratio}"
 
@@ -460,7 +598,9 @@ def grd_trend(
     da: xr.DataArray,
     coords: list = ["x", "y", "z"],
     deg: int = 1,
-    plot_all: bool = False,
+    plot: bool = False,
+    plot_type="pygmt",
+    **kwargs,
 ):
     """
     Fit an arbitrary order trend to a grid and use it to detrend.
@@ -473,8 +613,10 @@ def grd_trend(
         coordinate names of the supplied grid, by default ['x', 'y', 'z']
     deg : int, optional
         trend order to use, by default 1
-    plot_all : bool, optional
+    plot : bool, optional
         plot the results, by default False
+    plot_type : str, by default "pygmt"
+        choose to plot results with pygmt or xarray
 
     Returns
     -------
@@ -482,10 +624,17 @@ def grd_trend(
         returns xr.DataArrays of the fitted surface, and the detrended grid.
     """
 
+    # convert grid to a dataframe
     df = vd.grid_to_table(da).astype("float64")
     df.dropna(inplace=True)
+
+    # define a trend
     trend = vd.Trend(degree=deg).fit((df[coords[0]], df[coords[1]]), df[coords[2]])
+
+    # fit a trend to the grid of degree: deg
     df["fit"] = trend.predict((df[coords[0]], df[coords[1]]))
+
+    # remove the trend from the data
     df["detrend"] = df[coords[2]] - df.fit
 
     info = get_grid_info(da)
@@ -507,39 +656,88 @@ def grd_trend(
         registration=registration,
     )
 
-    if plot_all is True:
-        fig, ax = plt.subplots(ncols=3, nrows=1, figsize=(20, 20))
-        da.plot(
-            ax=ax[0],
-            robust=True,
-            cmap="viridis",
-            cbar_kwargs={
-                "orientation": "horizontal",
-                "anchor": (1, 1.8),
-                "label": "test",
-            },
-        )
-        ax[0].set_title("Input grid")
-        fit.plot(
-            ax=ax[1],
-            robust=True,
-            cmap="viridis",
-            cbar_kwargs={"orientation": "horizontal", "anchor": (1, 1.8)},
-        )
-        ax[1].set_title(f"Trend order {deg}")
-        detrend.plot(
-            ax=ax[2],
-            robust=True,
-            cmap="viridis",
-            cbar_kwargs={"orientation": "horizontal", "anchor": (1, 1.8)},
-        )
-        ax[2].set_title("Detrended")
-        for a in ax:
-            a.set_xticklabels([])
-            a.set_yticklabels([])
-            a.set_xlabel("")
-            a.set_ylabel("")
-            a.set_aspect("equal")
+    if plot is True:
+        if plot_type == "xarray":
+            fig, ax = plt.subplots(ncols=3, nrows=1, figsize=(20, 20))
+            da.plot(
+                ax=ax[0],
+                robust=True,
+                cmap="viridis",
+                cbar_kwargs={
+                    "orientation": "horizontal",
+                    "anchor": (1, 1.8),
+                    "label": "test",
+                },
+            )
+            ax[0].set_title("Input grid")
+            fit.plot(
+                ax=ax[1],
+                robust=True,
+                cmap="viridis",
+                cbar_kwargs={"orientation": "horizontal", "anchor": (1, 1.8)},
+            )
+            ax[1].set_title(f"Trend order {deg}")
+            detrend.plot(
+                ax=ax[2],
+                robust=True,
+                cmap="viridis",
+                cbar_kwargs={"orientation": "horizontal", "anchor": (1, 1.8)},
+            )
+            ax[2].set_title("Detrended")
+            for a in ax:
+                a.set_xticklabels([])
+                a.set_yticklabels([])
+                a.set_xlabel("")
+                a.set_ylabel("")
+                a.set_aspect("equal")
+
+        elif plot_type == "pygmt":
+            cmap = kwargs.get("cmap", "plasma")
+            coast = kwargs.get("coast", True)
+            inset = kwargs.get("inset", True)
+            inset_pos = kwargs.get("inset_pos", "BL")
+            origin_shift = kwargs.get("origin_shift", "yshift")
+            fit_label = kwargs.get("fit_label", f"fitted trend (order {deg})")
+            input_label = kwargs.get("input_label", "input grid")
+            title = kwargs.get("title", "Detrending a grid")
+            detrended_label = kwargs.get("detrended_label", "detrended")
+
+            fig = maps.plot_grd(
+                detrend,
+                cmap=cmap,
+                grd2cpt=True,
+                coast=coast,
+                cbar_label=detrended_label,
+                **kwargs,
+            )
+
+            fig = maps.plot_grd(
+                fit,
+                fig=fig,
+                cmap=cmap,
+                grd2cpt=True,
+                coast=coast,
+                cbar_label=fit_label,
+                inset=inset,
+                inset_pos=inset_pos,
+                origin_shift=origin_shift,
+                **kwargs,
+            )
+
+            fig = maps.plot_grd(
+                da,
+                fig=fig,
+                cmap=cmap,
+                grd2cpt=True,
+                coast=coast,
+                cbar_label=input_label,
+                title=title,
+                origin_shift=origin_shift,
+                **kwargs,
+            )
+
+            fig.show()
+
     return fit, detrend
 
 
@@ -571,12 +769,17 @@ def grd_compare(
         shapefile filename to use to mask the grids for setting the color range.
     robust : bool
         use xarray robust color lims instead of min and max, by default is False.
+    region : Union[str, np.ndarray]
+        choose a specific region to compare.
+    rmse_in_title: bool
+        add the RMSE to the title, by default is True.
     Returns
     -------
-    xr.DataArray
-        the result of da1 - da2
+    list
+        list of xr.DataArrays: (diff, resampled grid1, resampled grid2)
     """
     shp_mask = kwargs.get("shp_mask", None)
+    region = kwargs.get("region", None)
 
     if isinstance(da1, str):
         da1 = xr.load_dataarray(da1)
@@ -584,147 +787,268 @@ def grd_compare(
     if isinstance(da2, str):
         da2 = xr.load_dataarray(da2)
 
-    da1_spacing = get_grid_info(da1)[0]
-    da2_spacing = get_grid_info(da2)[0]
+    # first cut the grids to save time on the possible resampling below
+    if region is not None:
+        da1 = pygmt.grdcut(
+            da1,
+            region=region,
+            verbose=kwargs.get("verbose", "e"),
+        )
+        da2 = pygmt.grdcut(
+            da2,
+            region=region,
+            verbose=kwargs.get("verbose", "e"),
+        )
 
+    # get minimum grid spacing of both grids
+    da1_spacing = float(get_grid_info(da1)[0])
+    da2_spacing = float(get_grid_info(da2)[0])
+
+    min_spacing = min(da1_spacing, da2_spacing)
+
+    if da1_spacing != da2_spacing:
+        print(
+            "grid spacings don't match, using smaller spacing",
+            f"({min_spacing}m).",
+        )
+
+    # get inside region of both grids
     da1_reg = get_grid_info(da1)[1]
     da2_reg = get_grid_info(da2)[1]
 
-    spacing = min(da1_spacing, da2_spacing)
-
     e = max(da1_reg[0], da2_reg[0])
     w = min(da1_reg[1], da2_reg[1])
     n = max(da1_reg[2], da2_reg[2])
     s = min(da1_reg[3], da2_reg[3])
 
-    region = [e, w, n, s]
+    sub_region = [e, w, n, s]
 
-    if (da1_spacing != da2_spacing) and (da1_reg != da2_reg):
-        print(
-            "grid spacings and regions dont match, using smaller spacing",
-            f"({spacing}m) and inner region.",
-        )
-        grid1 = pygmt.grdsample(
-            da1,
-            spacing=spacing,
-            region=region,
-            registration="p",
-            verbose="e",
-        )
-        grid2 = pygmt.grdsample(
-            da2,
-            spacing=spacing,
-            region=region,
-            registration="p",
-            verbose="e",
-        )
-    elif da1_spacing != da2_spacing:
-        print("grid spacings dont match, using smaller spacing of supplied grids")
-        grid1 = pygmt.grdsample(
-            da1,
-            spacing=spacing,
-            registration="p",
-            verbose="e",
-        )
-        grid2 = pygmt.grdsample(
-            da2,
-            spacing=spacing,
-            registration="p",
-            verbose="e",
-        )
-    elif da1_reg != da2_reg:
-        print("grid regions dont match, using inner region of supplied grids")
-        grid1 = pygmt.grdcut(
-            da1,
-            region=region,
-            verbose="e",
-        )
-        grid2 = pygmt.grdcut(da2, region=region, verbose="e")
+    if da1_reg != da2_reg:
+        print(f"grid regions dont match, using inner region {sub_region}")
+
+    # use registration from first grid, or from kwarg
+    if kwargs.get("registration", None) is None:
+        registration = get_grid_info(da1)[4]
     else:
-        print("grid regions and spacing match")
-        grid1 = da1
-        grid2 = da2
+        registration = kwargs.get("registration", None)
+
+    grid1 = fetch.resample_grid(
+        da1,
+        spacing=min_spacing,
+        region=sub_region,
+        registration=registration,
+        verbose=kwargs.get("verbose", "e"),
+    )
+
+    grid2 = fetch.resample_grid(
+        da2,
+        spacing=min_spacing,
+        region=sub_region,
+        registration=registration,
+        verbose=kwargs.get("verbose", "e"),
+    )
 
     dif = grid1 - grid2
 
     # get individual grid min/max values (and masked values if shapefile is provided)
     grid1_cpt_lims = get_min_max(grid1, shp_mask)
     grid2_cpt_lims = get_min_max(grid2, shp_mask)
-    diff_cpt_lims = get_min_max(dif, shp_mask)
+
+    # if kwarg supplied, reset diff_maxabs
+    diff_maxabs = kwargs.get("diff_maxabs", vd.maxabs(get_min_max(dif, shp_mask)))
+
+    diff_lims = kwargs.get("diff_lims", (-diff_maxabs, diff_maxabs))
 
     # get min and max of both grids together
     vmin = min((grid1_cpt_lims[0], grid2_cpt_lims[0]))
     vmax = max(grid1_cpt_lims[1], grid2_cpt_lims[1])
 
     if plot is True:
+        title = kwargs.get("title", "Comparing Grids")
+        if kwargs.get("rmse_in_title", True) is True:
+            title += f", RMSE: {round(RMSE(dif),kwargs.get('RMSE_decimals', 2))}"
+
         if plot_type == "pygmt":
+            fig_height = kwargs.get("fig_height", 10)
+            coast = kwargs.get("coast", True)
+            origin_shift = kwargs.get("origin_shift", "xshift")
+            cmap = kwargs.get("cmap", "viridis")
+            subplot_labels = kwargs.get("subplot_labels", False)
+
+            new_kwargs = {
+                kw: kwargs[kw]
+                for kw in kwargs
+                if kw
+                not in [
+                    "cmap",
+                    "region",
+                    "coast",
+                    "title",
+                    "cpt_lims",
+                    "fig_height",
+                    "inset",
+                    "inset_pos",
+                ]
+            }
+
+            diff_kwargs = {
+                kw: new_kwargs[kw]
+                for kw in new_kwargs
+                if kw
+                not in [
+                    "reverse_cpt",
+                    "cbar_label",
+                ]
+            }
+
             fig = maps.plot_grd(
                 grid1,
-                cmap="viridis",
-                plot_regin=region,
-                coast=True,
-                cbar_label=kwargs.get("grid1_name", "grid 1"),
+                cmap=cmap,
+                region=region,
+                coast=coast,
+                title=kwargs.get("grid1_name", "grid 1"),
                 cpt_lims=(vmin, vmax),
+                fig_height=fig_height,
+                **new_kwargs,
             )
-            fig = maps.plot_grd(
-                grid2,
-                cmap="viridis",
-                plot_regin=region,
-                coast=True,
-                cbar_label=kwargs.get("grid2_name", "grid 2"),
-                cpt_lims=(vmin, vmax),
-                origin_shift="xshift",
-                fig=fig,
-            )
+            if subplot_labels is True:
+                fig.text(
+                    position="TL",
+                    justify="BL",
+                    text="a)",
+                    font=kwargs.get("label_font", "26p,Helvetica,black"),
+                    offset="j0/.3",
+                    no_clip=True,
+                )
             fig = maps.plot_grd(
                 dif,
-                cmap="polar",
-                plot_regin=region,
-                coast=True,
+                cmap=kwargs.get("diff_cmap", "balance+h0"),
+                region=region,
+                coast=coast,
+                origin_shift=origin_shift,
                 cbar_label="difference",
-                cpt_lims=diff_cpt_lims,
-                origin_shift="xshift",
+                cpt_lims=diff_lims,
                 fig=fig,
+                title=title,
+                inset=kwargs.get("inset", True),
+                inset_pos=kwargs.get("inset_pos", "TL"),
+                fig_height=fig_height,
+                **diff_kwargs,
             )
+            if subplot_labels is True:
+                fig.text(
+                    position="TL",
+                    justify="BL",
+                    text="b)",
+                    font=kwargs.get("label_font", "26p,Helvetica,black"),
+                    offset="j0/.3",
+                    no_clip=True,
+                )
+            fig = maps.plot_grd(
+                grid2,
+                cmap=cmap,
+                region=region,
+                coast=coast,
+                origin_shift=origin_shift,
+                fig=fig,
+                title=kwargs.get("grid2_name", "grid 2"),
+                cpt_lims=(vmin, vmax),
+                fig_height=fig_height,
+                **new_kwargs,
+            )
+            if subplot_labels is True:
+                fig.text(
+                    position="TL",
+                    justify="BL",
+                    text="c)",
+                    font=kwargs.get("label_font", "26p,Helvetica,black"),
+                    offset="j0/.3",
+                    no_clip=True,
+                )
+
             fig.show()
 
         elif plot_type == "xarray":
             if kwargs.get("robust", False) is True:
                 vmin, vmax = None, None
+            cmap = kwargs.get("cmap", "viridis")
+
+            sub_width = 5
+            nrows, ncols = 1, 3
+            # setup subplot figure
+            fig, ax = plt.subplots(
+                nrows=nrows,
+                ncols=ncols,
+                figsize=(sub_width * ncols, sub_width * nrows),
+            )
 
-            fig, ax = plt.subplots(ncols=3, nrows=1, figsize=(20, 20))
             grid1.plot(
                 ax=ax[0],
-                cmap="viridis",
+                cmap=cmap,
                 vmin=vmin,
                 vmax=vmax,
                 robust=True,
-                cbar_kwargs={"orientation": "horizontal", "anchor": (1, 1.8)},
-            )
-            grid2.plot(
-                ax=ax[1],
-                cmap="viridis",
-                vmin=vmin,
-                vmax=vmax,
-                robust=True,
-                cbar_kwargs={"orientation": "horizontal", "anchor": (1, 1.8)},
+                cbar_kwargs={
+                    "orientation": "horizontal",
+                    "anchor": (1, 1),
+                    "fraction": 0.05,
+                    "pad": 0.04,
+                },
             )
+            ax[0].set_title(kwargs.get("grid1_name", "grid 1"))
+
             dif.plot(
-                ax=ax[2],
-                vmin=diff_cpt_lims[0],
-                vmax=diff_cpt_lims[1],
+                ax=ax[1],
+                vmin=diff_lims[0],
+                vmax=diff_lims[1],
                 robust=True,
-                cmap="RdBu_r",
-                cbar_kwargs={"orientation": "horizontal", "anchor": (1, 1.8)},
+                cmap=kwargs.get("diff_cmap", "RdBu_r"),
+                cbar_kwargs={
+                    "orientation": "horizontal",
+                    "anchor": (1, 1),
+                    "fraction": 0.05,
+                    "pad": 0.04,
+                },
             )
+
+            ax[1].set_title(title)
+
+            grid2.plot(
+                ax=ax[2],
+                cmap=cmap,
+                vmin=vmin,
+                vmax=vmax,
+                robust=True,
+                cbar_kwargs={
+                    "orientation": "horizontal",
+                    "anchor": (1, 1),
+                    "fraction": 0.05,
+                    "pad": 0.04,
+                },
+            )
+            ax[2].set_title(kwargs.get("grid2_name", "grid 2"))
+
             for a in ax:
                 a.set_xticklabels([])
                 a.set_yticklabels([])
                 a.set_xlabel("")
                 a.set_ylabel("")
                 a.set_aspect("equal")
+                if kwargs.get("points", None) is not None:
+                    a.plot(kwargs.get("points").x, kwargs.get("points").y, "k+")
+                if kwargs.get("show_region", None) is not None:
+                    show_region = kwargs.get("show_region", None)
+                    a.add_patch(
+                        mpl.patches.Rectangle(
+                            xy=(show_region[0], show_region[2]),
+                            width=(show_region[1] - show_region[0]),
+                            height=(show_region[3] - show_region[2]),
+                            linewidth=1,
+                            fill=False,
+                        )
+                    )
 
-    return dif, grid1, grid2
+    return (dif, grid1, grid2)
 
 
 def make_grid(
@@ -753,7 +1077,7 @@ def make_grid(
         Returns a xr.DataArray with 1 variable of constant value.
     """
     coords = vd.grid_coordinates(region=region, spacing=spacing, pixel_register=True)
-    data = np.ones_like(coords[0]) + value
+    data = np.ones_like(coords[0]) * value
     grid = vd.make_xarray_grid(coords, data, dims=["y", "x"], data_names=name)
     return grid
 
@@ -789,10 +1113,8 @@ def raps(
     spacing : float
         grid spacing if input is not a grid
     """
-    try:
-        import seaborn as sns
-    except ImportError:
-        print("package `seaborn` not installed")
+    if not _has_seaborn:
+        raise ImportError("seaborn is required for this function.")
 
     region = kwargs.get("region", None)
     spacing = kwargs.get("spacing", None)
@@ -856,7 +1178,7 @@ def raps(
                 delimiter="\t",
                 names=("wavelength", "power", "stdev"),
             )
-            ax = sns.lineplot(raps.wavelength, raps.power, label=j, palette="viridis")
+            ax = sns.lineplot(x=raps.wavelength, y=raps.power, label=j)
             ax = sns.scatterplot(x=raps.wavelength, y=raps.power)
             ax.set_xlabel("Wavelength (km)")
             ax.set_ylabel("Radially Averaged Power ($mGal^{2}km$)")
@@ -902,10 +1224,10 @@ def coherency(grids: list, label: str, **kwargs):
     spacing : float
         grid spacing if input is pd.DataFrame
     """
-    try:
-        import seaborn as sns
-    except ImportError:
-        print("package `seaborn` not installed")
+
+    if not _has_seaborn:
+        raise ImportError("seaborn is required for this function.")
+
     region = kwargs.get("region", None)
     spacing = kwargs.get("spacing", None)
 
@@ -1007,27 +1329,35 @@ def coherency(grids: list, label: str, **kwargs):
     """
 
 
-def square_subplots(n):
+def square_subplots(n: int):
     """
     From https://github.com/matplotlib/grid-strategy/blob/master/src/grid_strategy/strategies.py # noqa
-    Return an arrangement of rows containing ``n`` axes that is as close to
-    square as looks good.
-    :param n:
-        The number of plots in the subplot
-    :return:
-        Returns a  :class:`tuple` of length ``nrows``, where each element
-        represents the number of plots in that row, so for example a 3 x 2
+    Calculate the number of rows and columns based on the total number of items (n) to
+    make an arrangement as close to square as looks good.
+
+    Parameters
+    ----------
+    n : int
+        The number of total plots in the subplot
+
+    Returns
+    -------
+    tuple
+        Returns a tuple in the format (number of rows, number of columns), so for example a 3 x 2
         grid would be represented as ``(3, 3)``, because there are 2 rows
         of length 3.
-    Example:
-    --------
-    .. code::
-        >>> GridStrategy.get_grid(7)
-        (2, 3, 2)
-        >>> GridStrategy.get_grid(6)
-        (3, 3)
     """
-    SPECIAL_CASES = {3: (2, 1), 5: (2, 3)}
+    SPECIAL_CASES = {
+        1: (1, 1),
+        2: (1, 2),
+        3: (2, 2),
+        4: (2, 2),
+        5: (2, 3),
+        6: (2, 3),
+        7: (3, 3),
+        8: (3, 3),
+        9: (3, 3),
+    }
     if n in SPECIAL_CASES:
         return SPECIAL_CASES[n]
 
@@ -1162,3 +1492,183 @@ def polygon_to_region(polygon: list):
     region = vd.get_region((df.x, df.y))
 
     return region
+
+
+def mask_from_polygon(
+    polygon: list,
+    invert: bool = False,
+    drop_nans: bool = False,
+    grid: Union[str, xr.DataArray] = None,
+    region: list = None,
+    spacing: int = None,
+    **kwargs,
+):
+    """
+    convert the output of `regions.draw_region` to a mask or use it to mask a grid
+
+    Parameters
+    ----------
+    polygon : list
+       list of polygon vertices
+    invert : bool, optional
+        reverse the sense of masking, by default False
+    drop_nans : bool, optional
+        drop nans after masking, by default False
+    grid : Union[str, xr.DataArray], optional
+        grid to mask, by default None
+    region : list, optional
+        region to create a grid if none is supplied, by default None
+    spacing : int, optional
+        spacing to create a grid if none is supplied, by default None
+
+    Returns
+    -------
+    xr.Dataarray
+        masked grid or mask grid with 1's inside the mask.
+    """
+
+    # convert drawn polygon into dataframe
+    df = shapes_to_df(polygon)
+    data_coords = (df.x, df.y)
+
+    # remove additional polygons
+    if df.shape_num.max() > 0:
+        print("supplied dataframe has multiple polygons, only using the first one.")
+        df = df[df.shape_num == 0]
+
+    # if grid given as filename, load it
+    if isinstance(grid, str):
+        grid = xr.load_dataarray(grid)
+        ds = grid.to_dataset()
+    elif isinstance(grid, xr.DataArray):
+        ds = grid.to_dataset()
+
+    # if no grid given, make a dummy one with supplied region and spacing
+    if grid is None:
+        coords = vd.grid_coordinates(
+            region=region,
+            spacing=spacing,
+            pixel_register=kwargs.get("pixel_register", False),
+        )
+        ds = vd.make_xarray_grid(
+            coords, np.ones_like(coords[0]), dims=("y", "x"), data_names="z"
+        )
+
+    masked = vd.convexhull_mask(
+        data_coords,
+        grid=ds,
+    ).z
+
+    # reverse the mask
+    if invert is True:
+        inverse = masked.isnull()
+        inverse = inverse.where(inverse != 0)
+        masked = inverse * ds.z
+
+    # drop nans
+    if drop_nans is True:
+        masked = masked.where(masked.notnull() == 1, drop=True)
+
+    return masked
+
+
+def change_reg(grid):
+    """
+    Use GMT grdedit to change the registration type in the metadata.
+
+    Parameters
+    ----------
+    grid : xr.DataArray
+        input grid to change the reg for.
+
+    Returns
+    -------
+    xr.DataArray
+        returns a dataarray with switch reg type.
+    """
+    with pygmt.clib.Session() as ses:
+        # store the input grid in a virtual file so GMT can read it from a dataarray
+        with ses.virtualfile_from_grid(grid) as f_in:
+            # send the output to a file so that we can read it
+            with pygmt.helpers.GMTTempFile(suffix=".nc") as tmpfile:
+                args = f"{f_in} -T -G{tmpfile.name}"
+                ses.call_module("grdedit", args)
+                f_out = pygmt.load_dataarray(tmpfile.name)
+    return f_out
+
+
+def grd_blend(
+    grid1: xr.DataArray,
+    grid2: xr.DataArray,
+    **kwargs,
+):
+    """
+    Use GMT grdblend to blend 2 grids into 1.
+
+    Parameters
+    ----------
+    grid1 : xr.DataArray
+        input grid to change the reg for.
+
+    grid2 : xr.DataArray
+        input grid to change the reg for.
+
+    Returns
+    -------
+    xr.DataArray
+        returns a blended dataarray.
+    """
+    with pygmt.clib.Session() as session:
+        with pygmt.helpers.GMTTempFile(suffix=".nc") as tmpfile:
+            # store the input grids in a virtual files so GMT can read it from
+            # dataarrays
+            file_context1 = session.virtualfile_from_grid(grid1)
+            file_context2 = session.virtualfile_from_grid(grid2)
+            with file_context1 as infile1, file_context2 as infile2:
+                # if (outgrid := kwargs.get("G")) is None:
+                #     kwargs["G"] = outgrid = tmpfile.name # output to tmpfile
+                args = f"{infile1} {infile2} -Cf -G{tmpfile.name}"
+                session.call_module(module="grdblend", args=args)
+    return pygmt.load_dataarray(infile1)  # if outgrid == tmpfile.name else None
+
+
+def get_fig_width(figure):
+    with pygmt.clib.Session() as session:
+        with pygmt.helpers.GMTTempFile() as tmpfile:
+            session.call_module("mapproject", f"-Ww ->{tmpfile.name}")
+            map_width = tmpfile.read().strip()
+    return float(map_width)
+
+
+def get_fig_height(figure):
+    with pygmt.clib.Session() as session:
+        with pygmt.helpers.GMTTempFile() as tmpfile:
+            session.call_module("mapproject", f"-Wh ->{tmpfile.name}")
+            map_height = tmpfile.read().strip()
+    return float(map_height)
+
+
+def GMT_str_to_list(region: list):
+    return "".join([str(x) + "/" for x in region])[:-1]
+
+
+def grd_mask(
+    df,
+    spacing,
+    region,
+    clobber="o",
+    values="0/0/1",
+    radius="0c",
+):
+    with pygmt.clib.Session() as ses:
+        # store the input grid in a virtual file so GMT can read it from a dataarray
+        with ses.virtualfile_from_data(x=df.x, y=df.y, z=df.z) as f_in:
+            # send the output to a file so that we can read it
+            with pygmt.helpers.GMTTempFile(suffix=".nc") as tmpfile:
+                args = (
+                    f"{f_in} -I{spacing} -R{str(GMT_str_to_list(region))}",
+                    f"-C{clobber} -N{values} -S{radius} -G{tmpfile.name}",
+                )
+                ses.call_module("grdmask", args)
+                f_out = pygmt.load_dataarray(tmpfile.name)
+    return f_out
